
/****************************************************************************/
/*                                                                          */
/* output example / test                                                    */
/*                                                                          */
/****************************************************************************/

#include <idsa.h>
#include <idsa_internal.h>

#define BUFFERSIZE 4096

int main()
{
  IDSA_CONNECTION *c;
  IDSA_EVENT *e;
  char buffer[BUFFERSIZE];
  int len;
  int result;

  c = idsa_open("test-service", NULL, 0);

  if (c) {

    e = idsa_event(c);

    if (e) {
      /* internal, only important if you want to debug protocol */
      idsa_event_dump(e, stderr);

      /* set fields individually instead of in one go with idsa_set or ids_scan */
      idsa_name(e, "test-event");
      idsa_scheme(e, "test-scheme");
      idsa_add_string(e, "test-field", "test-value");
      idsa_add_integer(e, "test-number", 42);

      /* internal, only important if you want to debug protocol */
      idsa_event_dump(e, stderr);

      /* display in various formats */
      fprintf(stderr, "Native format:\n");
      len = idsa_print_native(e, buffer, BUFFERSIZE);
      if (len > 0) {
	fwrite(buffer, 1, len, stderr);
      }
      fprintf(stderr, "Syslog format:\n");
      len = idsa_print_syslog(e, buffer, BUFFERSIZE);
      if (len > 0) {
	fwrite(buffer, 1, len, stderr);
      }
      fprintf(stderr, "CSV format:\n");
      len = idsa_print_csv(e, buffer, BUFFERSIZE);
      if (len > 0) {
	fwrite(buffer, 1, len, stderr);
      }
      fprintf(stderr, "ULM format:\n");
      len = idsa_print_ulm(e, buffer, BUFFERSIZE);
      if (len > 0) {
	fwrite(buffer, 1, len, stderr);
      }
      fprintf(stderr, "TULM format:\n");
      len = idsa_print_tulm(e, buffer, BUFFERSIZE);
      if (len > 0) {
	fwrite(buffer, 1, len, stderr);
      }
      result = idsa_log(c, e);
      if (result == IDSA_L_ALLOW) {
	fprintf(stderr, "RESULT: allow\n");
      } else {
	fprintf(stderr, "RESULT: deny\n");
      }

    }
    idsa_close(c);
  }
  return 0;
}
