#ifndef _IDSA_BRANCH_
#define _IDSA_BRANCH_

#include <idsa_internal.h>

#include "types.h"
#include "rule.h"
#include "table.h"
#include "queue.h"
#include "mod.h"

/* numeric comparisons are made against type when deciding to reorder */
/* lower values are assumed to be cheaper to evaluate */

#define BRANCH_TYPE_TABLE 0x01
#define BRANCH_TYPE_QUEUE 0x02
#define BRANCH_TYPE_MOD   0x03

struct union_branch{
  int b_compares; /* total number of compares performed (poor man's cost) */
  int b_runs; /* number of runs */

  int b_type;
  RULE_BODY *b_first; /* first rule which needs this */
  RULE_BODY *b_last;  /* last rule which needs this */

  union{
    TABLE_HEAD *b_table;
    QUEUE_MATCH *b_queue;
    MOD_PIPE *b_mod;
  } b_un;

  struct union_branch *b_prev; /* doubly linked list of branches */
  struct union_branch *b_next;
};

UNION_BRANCH *branch_new(RULE_BODY *r, int t);
void branch_free(UNION_BRANCH *b);
void branch_append(STATE_SET *s, UNION_BRANCH *b);

int branch_trigger(UNION_BRANCH *b, IDSA_EVENT *e);

void branch_dump(STATE_SET *s, FILE *fp);

UNION_BRANCH *branch_reorder(STATE_SET * s, UNION_BRANCH * b);

#endif
