#ifndef _MEX_
#define _MEX_

/* stupid little tokenizer, but is extensible at run time */

#define MEX_BUFFER 4096
#define MEX_CHAR    256

#include "types.h"

struct mex_token{ /* token returned to caller */
  int t_id; /* user assigned id */
  int t_type; /* keyword, string, etc */
  char *t_buf; /* copied payload */
  int t_len; /* length of said payload */
  int t_line; /* line on which it occurs */
  MEX_TOKEN *t_next; /* linked list for internal user only */
};

struct mex_key{ /* tree to do keyword lookup */
  char *k_name; /* string to compare against */
  int k_id; /* id to be copied into t_id */
  struct mex_key *k_left,*k_right; /* children */
};

struct mex{ /* state */
  int m_fd; /* input stream */
  int m_eof; /* encountered eof */
  int m_error; /* saved error */
  int m_line; /* line number */
  unsigned char m_buf[MEX_BUFFER]; /* read buffer */
  int m_read; /* number of bytes read into buffer */
  int m_lexed; /* number of read bytes converted to tokens */
  MEX_TOKEN *m_token; /* list of available tokens */
  MEX_TOKEN *m_garbage; /* list of returned tokens for garbage collection */
  int m_keychars[MEX_CHAR]; /* lookup table for single character keywords */
  MEX_KEY *m_keywords; /* tree for all other keywords */
};

#define MEX_KEYWORD   0x01
#define MEX_WORD      0x02
#define MEX_STRING    0x03

MEX *mex_fdopen(int fd); /* create mex instance from fd */
MEX *mex_open(char *fname); /* create instance from file name */
int mex_close(MEX *m); /* zap the entire thing */

int mex_keychar(MEX *m, unsigned char c, int k); /* allocate key character */
int mex_keyword(MEX *m, unsigned char *s, int k); /* key word */

MEX_TOKEN *mex_get(MEX *m); /* grab the next token */
void mex_unget(MEX *m, MEX_TOKEN *t); /* oops, we did not need it */

char *mex_error(MEX *m); /* display error string */
void mex_dump(MEX *m, FILE *f); /* for debugging */

#endif
